-----------------------------------------------------------------------------------------------
-- Client Lua Script for ReIA
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ReIA Module Definition
-----------------------------------------------------------------------------------------------
local ReIA = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ReIA:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.golgoxAlert = false;
	self.golgoxAlerted = false;

    return o
end

function ReIA:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ReIA OnLoad
-----------------------------------------------------------------------------------------------
function ReIA:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ReIA.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ReIA OnDocLoaded
-----------------------------------------------------------------------------------------------
function ReIA:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ReIAForm", nil, self)
		self.wndAlert = Apollo.LoadForm(self.xmlDoc, "alert", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("reia", "OnReIAOn", self)
		Apollo.RegisterEventHandler("TargetUnitChanged", "OnChangedTarget", self)
		Apollo.RegisterEventHandler("WindowManagementReady", "OnWindowManagementReady", self)

		self.timer = ApolloTimer.Create(0.100, true, "OnTimer", self)

		-- Do additional Addon initialization here
		self.wndMain:Invoke()
		self.timer:Stop();
		self.wndMain:SetOpacity(0, 100);
		
		--We don't clear targets on reloadUI so check if target and if yes then set IA
		if GameLib.GetTargetUnit() then
			self.target = GameLib.GetTargetUnit();
			
			--If the target isn't a friend, then get things going
			if not self:IsFriendly(self.target) then
				self.targetArmor = self.target:GetInterruptArmorValue();
				self.timer:Start();
			end	
		end
		
	end
end

-----------------------------------------------------------------------------------------------
-- ReIA Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
--Determine friends
function ReIA:IsFriendly(unit)
	self.player = GameLib.GetPlayerUnit();
	local disposition = unit:GetDispositionTo(self.player);
	
	if disposition == 2 then
		return true
	else 
		return false
	end
end

-- on SlashCommand "/reia"
function ReIA:OnReIAOn(cmd, arg)
	if arg == "" then
		Print("[ReIA] No Arguments Given");
		Print("[ReIA] Use '/reia' with the following:");
		Print("[ReIA] unlock - allows movement");
		Print("[ReIA] lock - locks in place");
		Print("[ReIA] [#] - will alert you for interrupts on Phageborn");
	end
	
	--Unlock
	if arg == "unlock" then
		self.wndMain:SetStyle("Moveable", true);
		self.wndMain:SetStyle("IgnoreMouse", false);
		self.wndMain:SetStyle("Noclip", false);
		self.wndMain:SetOpacity(100, 5);
	end
	
	--lock
	if arg == "lock" then
		self.wndMain:SetOpacity(0, 100);
		self.wndMain:SetStyle("Moveable", false);
		self.wndMain:SetStyle("IgnoreMouse", true);
		self.wndMain:SetStyle("Noclip", true);
	end
	
	--number
	if tonumber(arg) then
		self.golgoxAlert = arg;
		Print("Interrupt alert on Golgox at " .. tostring(arg) .. " IA.");
	end

end

--on target changed
function ReIA:OnChangedTarget()
	--Set target to new selection
	self.target = GameLib.GetTargetUnit()
		
	--If the target doesn't exist, or is friendly then do not do anything
	if (self.target == nil or self:IsFriendly(self.target)) then
		self.timer:Stop();
		self.wndMain:SetOpacity(0, 100);
		self.wndAlert:Close();
		return
	end
	
	--Grab current IA value
	self.targetArmor = self.target:GetInterruptArmorValue();
	
	--Update UI
	self.wndMain:FindChild("iaText"):SetText(tostring(self.targetArmor));
	
	--Start timer
	self.timer:Start();
	
end

-- on timer
function ReIA:OnTimer()
	--We have our target, and they are bad guys!
	if self.target ~= nil and not self:IsFriendly(self.target) then
		local newArmor = self.target:GetInterruptArmorValue();
		if newArmor ~= self.targetArmor then
			self.targetArmor = newArmor;
			self.wndMain:FindChild("iaText"):SetText(tostring(self.targetArmor));
		end
	
		--Boss specific checks go to their own functions
		if self.golgoxAlert then
			self:isGolgox();
		end
		
		--Set our opacity
		self.wndMain:SetOpacity(100, 5);
	end
end

--Golgox checkerino
function ReIA:isGolgox()
	--If it's golgox
	if self.target:GetName() == "Golgox the Lifecrusher" then
		if tostring(self.targetArmor) == tostring(self.golgoxAlert) and not self.golgoxAlerted then
			self.wndAlert:Invoke();
			self:Alert();
			self.golgoxAlerted = true;
		else if self.golgoxAlerted and tostring(self.targetArmor) ~= tostring(self.golgoxAlert) then
			--Need to reset this in the event of more than 1 hammer phase
			self.golgoxAlerted = false;
			self.wndAlert:Close();
		end
		end
	else
		self.wndAlert:Close();
	end
end

--Interrupt alert
function ReIA:Alert()
	Sound.Play(203);
end

--Save position across sessions
function ReIA:OnWindowManagementReady()
	Event_FireGenericEvent("WindowManagementAdd", {wnd = self.wndMain, strName = "reiaMain"})
end
-----------------------------------------------------------------------------------------------
-- ReIA Instance
-----------------------------------------------------------------------------------------------
local ReIAInst = ReIA:new()
ReIAInst:Init()
